# Copyright (C) 1999, 2000 Jay Beale
# assisted by Peter Watkins
# Licensed under the GNU General Public License

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#
# NOTE   : This module is a stand-in for a more featureful model that uses all
#          the functionality of Peter's new firewall script.  Basically, this
#          doesn't handle dhcp-type connections nearly as well.  This is JJB's
#          temporary stand-in for a more featureful version...
#

package Bastille::IPCHAINS;

use Bastille::API;
@ENV="";
undef(%ENV);
$ENV{PATH}="";
$ENV{CDPATH}=".";
$ENV{BASH_ENV}="";

# @ipchainsOptions contains information about the variables this script sets the value of
# in the final firewall script. Each item in @ipchainsOptions is a hash describing a certain
# environment variable. 
#
# Each such hash should have the following name-value pairs:
#	varname		the actual variable name for the /bin/sh firewall script
#	default		the default/suggested value for that variable (in case nothing in the config)
#	stanza		where this is used in the template script
#	configname	the name for the user's choice for this question, as read
#			from the "config" script


@ipchainsOptions = (

	{
	'varname' => "DNS_SERVERS",
	'default' => "",
	'stanza' => "0",
	'configname' => 'ip_s_dns',
	},

	{
	'varname' => "TRUSTED_IFACES",
	'default' => "lo",
	'stanza' => "1",
	'configname' => 'ip_s_trustiface',
	},

	{
	'varname' => "PUBLIC_IFACES",
	'default' => "eth+ ppp+",
	'stanza' => "1",
	'configname' => 'ip_s_publiciface',
	},

	{
	'varname' => "INTERNAL_IFACES",
	'default' => "",
	'stanza' => "1",
	'configname' => 'ip_s_internaliface',
	},

	{
	'varname' => "TCP_AUDIT_SERVICES",
	'default' => "telnet ftp imap pop-3 finger sunrpc exec login linuxconf ssh",
	'stanza' => "2",
	'configname' => 'ip_s_tcpaudit',
	},

	{
	'varname' => "UDP_AUDIT_SERVICES",
	'default' => "31337",
	'stanza' => "2",
	'configname' => 'ip_s_udpaudit',
	},

	{
	'varname' => "ICMP_AUDIT_TYPES",
	'default' => "",
	'stanza' => "2",
	'configname' => 'ip_s_icmpaudit',
	},

	{
	'varname' => "TCP_PUBLIC_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_publictcp',
	},

	{
	'varname' => "UDP_PUBLIC_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_publicudp',
	},

	{
	'varname' => "TCP_INTERNAL_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_internaltcp',
	},

	{
	'varname' => "UDP_INTERNAL_SERVICES",
	'default' => "",
	'stanza' => "3",
	'configname' => 'ip_s_internaludp',
	},

	{
	'varname' => "FORCE_PASV_FTP",
	'default' => "N",
	'stanza' => "4",
	'configname' => 'ip_s_passiveftp',
	},

	{
	'varname' => "TCP_BLOCKED_SERVICES",
	'default' => "6000:6020",
	'stanza' => "5",
	'configname' => 'ip_s_tcpblock',
	},

	{
	'varname' => "UDP_BLOCKED_SERVICES",
	'default' => "",
	'stanza' => "5",
	'configname' => 'ip_s_udpblock',
	},

	{
	'varname' => "ICMP_ALLOWED_TYPES",
	'default' => "destination-unreachable echo-reply time-exceeded",
	'stanza' => "5",
	'configname' => 'ip_s_icmpallowed',
	},

	{
	'varname' => "ENABLE_SRC_ADDR_VERIFY",
	'default' => "Y",
	'stanza' => "6",
	'configname' => 'ip_s_srcaddr',
	},

	{
	'varname' => "IP_MASQ_NETWORK",
	'default' => "",
	'stanza' => "7",
	'configname' => 'ip_s_ipmasq',
	},

	{
	'varname' => "IP_MASQ_MODULES",
	'default' => "ftp raudio vdolive",
	'stanza' => "7",
	'configname' => 'ip_s_kernelmasq',
	},

	{
	'varname' => "REJECT_METHOD",
	'default' => "DENY",
	'stanza' => "8",
	'configname' => 'ip_s_rejectmethod',
	},

	{
	'varname' => "DHCP_IFACES",
	'default' => "",
	'stanza' => "9",
	'configname' => 'ip_s_dhcpiface',
	},

	{
	'varname' => "NTP_SERVERS",
	'default' => "",
	'stanza' => "10",
	'configname' => 'ip_s_ntpsrv',
	},

	{
	'varname' => "ICMP_OUTBOUND_DISABLED_TYPES",
	'default' => "destination-unreachable time-exceeded",
	'stanza' => "11",
	'configname' => 'ip_s_icmpout',
	},

	);

# JJB's fix -- get this to work when they choose the ip_b_ questions instead.

if ( $GLOBAL_CONFIG{IPChains}{ip_advnetwork} eq "N" ) {
    
    for (my $loop=0; $loop < scalar(@ipchainsOptions) ; ++$loop) {
	if ( $ipchainsOptions[$loop]{configname} =~ /^ip_s_(.*)$/ ) {
	    $ipchainsOptions[$loop]{configname} = "ip_b_" . $1;
	}
    }
}

# This is where the code uses Jay's subroutine, temporarily, to get this out the door faster...

#&InstallScript;
&InstallScript_simpler;


sub ipchainsAddNetmask {
	# adds netmask info to plain dotted-quad IP addresses
	my ( $phrase ) = @_;
	if ( $phrase =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ ) {
		# looks like a plain IPv4 dot-quad address
		if ( $phrase eq '0.0.0.0' ) {
			$phrase .= '/0.0.0.0';
		} elsif ( $phrase =~ /\.0\.0\.0$/ ) {
			$phrase .= '/255.0.0.0';
		} elsif ( $phrase =~ /\.0\.0$/ ) {
			$phrase .= '/255.255.0.0';
		} elsif ( $phrase =~ /\.0$/ ) {
			$phrase .= '/255.255.255.0';
		} else {
			$phrase .= '/255.255.255.255';
		}
	} 
	return $phrase;
}

sub ipchainsCleanItemList {
	# removes spaces, cleans up whitespace in string
	my ( $list ) = @_;
	my ( @items, $one );
	$list =~ s/\,/ /g;
	$list =~ s/[^a-zA-Z0-9\-\_\s\.\:\+\/]//eg;	# remove unexpected chars
	$list =~ s/[\n\r\t]/ /g;
	while ( $list =~ /  / ) { $list =~ s/  / /g;}
	@items = split(/ /, $list);
	$list = '';
	foreach $one ( @items ) {
		$list .= ' '.&ipchainsAddNetmask($one);
	}
	$list =~ s/^.//;
	return $list;
}

sub InstallScript_simpler {

    # This is JJB's (possibly temporary) modification to IPCHAINS.pm to port to 1.1 architecture.

    my (@varArray,$loop);

    my $configPrefix="IPChains";

    my $firewall_init_script = $GLOBAL_DIR{"initd"} . "/bastille-firewall";
    my $virgin_init_script = $GLOBAL_BDIR{"home"} . "/bastille-firewall";

    # more debug code
#    print "1: $firewall_init_script \n2: $virgin_init_script\n";

    # If the firewall script has not been installed, put it in place.
    unless ( -e $firewall_init_script ) {
	&B_cp($virgin_init_script,$firewall_init_script);
	&B_chmod(0500,$firewall_init_script);
    }

    # Now, iterate through the answers filling in the script...
    for ($loop=0; $loop < scalar(@ipchainsOptions); ++$loop) {
	
	# Build the @varArray to hold text to be inserted.

	my $ans = &ipchainsCleanItemList($GLOBAL_CONFIG{$configPrefix}{$ipchainsOptions[$loop]{configname}});
	my $variable = $ipchainsOptions[$loop]{varname};
	my $assignment = $variable . "=\"$ans\"\n";

	## DEBUG CODE!!!
	print "Setting $variable to \"$ans\"\n";

	my $pattern = "^${variable}\s*=";
	&B_replace_line($firewall_init_script,$pattern,$assignment);

    } 

}


sub InstallScript {

   my ( @varArray, $position, $status, $mode, $model, $ok, $ans, $loop );

   my $insertPrefix = "\#\n\# Inserted by Bastille install script:\n";
   my $configPrefix = 'IPChains';
   my $firewallExists = 1;

      &B_open(*ORIG_FIREWALL,"bastille-firewall");

	# Ask the user about the different options, in order
	for ($loop = 0; $loop < scalar(@ipchainsOptions); ++$loop) {
		# in this we build our @varArray to hold the text to be inserted
		# in the /bin/sh script after the different stanzas
		$ans = &ipchainsCleanItemList($GLOBAL_CONFIG{$configPrefix}{$ipchainsOptions[$loop]{configname}});
		$varArray[$ipchainsOptions[$loop]{stanza}] .= 
			"$ipchainsOptions[$loop]{varname}\=\"" . $ans. "\"\n";	
		print "Would set $ipchainsOptions[$loop]{varname} to \"$ans\"\n";		
	}
	return;

	# ----------------------------------------------------------------------------------------
	# should check if firewall script exists, if not, create it and set $firewallExists to 0
	# ----------------------------------------------------------------------------------------


	# ----------------------------------------------------------------------------------------
	# insert the new values: should use B_insertChunk/B_replaceChunk()

      &B_open(*FIREWALL,">/etc/rc.d/init.d/bastille-firewall");
      # $status is used to note whether we should look for a blank
      # line to add variable settings from @varArray
      $status = 'clear';
      while ($line=<ORIG_FIREWALL>) {
	 if ( $line =~ /^\# ([0-9]{1,}?)\)/ ) {
		# Found a line beginning with "# nn)", the first line of
		# stanza "nn", so we should start looking for a blank
		# line to add our custom settings
		$position = $1;
		$status = 'wait';
	 }
	 if ( ($line =~ /^\s{0,}$/) && ($status eq 'wait') ) {
		# just passed a config stanza, add the variable-setting lines
         	&B_print(*FIREWALL,$insertPrefix.$varArray[$position]);
		# back to normal status
		$status = 'clear';
	 }
         &B_print(*FIREWALL,$line);
      }

      &B_close(*FIREWALL);
      &B_close(*ORIG_FIREWALL);
	# ----------------------------------------------------------------------------------------

	# Do this only if we just created the firewall script
	if ( $firewallExists == 0 ) {
	      # There's no good reason for local users to read the firewall config,
	      # is there?
	      &B_chmod(0500, '/etc/rc.d/init.d/bastille-firewall');
	}


	# ----------------------------------------------------------------------------------------
      # Install the reset script that ifup-local runs IF IT DOESN'T ALREADY EXIST
      &B_open(*RESET_TEMPLATE,"bastille-firewall-reset");
      &B_open(*RESET_SCRIPT,">/sbin/bastille-firewall-reset");
      while ($line=<RESET_TEMPLATE>) {
         &B_print(*RESET_SCRIPT,$line);
      }
      &B_close(*RESET_TEMPLATE);
      &B_close(*RESET_SCRIPT);
      &B_chmod(0500, '/sbin/bastille-firewall-reset');
	# ----------------------------------------------------------------------------------------


	# ----------------------------------------------------------------------------------------
	# Need to do something else for RHAT 6.2+ (pump 0.7.8+), specifically add
	#	script /sbin/bastille-firewall-reset 
	# to /etc/pump.conf
	
	# If we're NOT using the newer pump, then we should add the schedule script
	
      # Install the pump scheduling script that ifup-local runs
      &B_open(*SCHEDULE_TEMPLATE,"bastille-firewall-schedule");
      &B_open(*SCHEDULE_SCRIPT,">/sbin/bastille-firewall-schedule");
      while ($line=<SCHEDULE_TEMPLATE>) {
         &B_print(*SCHEDULE_SCRIPT,$line);
      }
      &B_close(*SCHEDULE_TEMPLATE);
      &B_close(*SCHEDULE_SCRIPT);
      &B_chmod(0500, '/sbin/bastille-firewall-schedule');
	# ----------------------------------------------------------------------------------------

      # Install the ifup-local script
      if ( (-e "/sbin/ifup-local") && (! -e "/sbin/ifup-local.pre-bastille") ) {
		# no pre-bastille yet; see if the current ifup-local
		# script is ours
		$looksLikeOurs = 0;
      		&B_open(*IFUP_CURRENT,"/sbin/ifup-local");
		while ($line=<IFUP_CURRENT>) {
			if ( $line =~ /\/sbin\/bastille\-firewall\-reset/ ) {
				# it looks like our script, no need
				# to make a pre-bastille copy
				$looksLikeOurs = 1;
			}
		}
		&B_close(*IFUP_CURRENT);
		if ( $looksLikeOurs == 0 ) {
 			# make copy of current ufup-local script that our 
			# ifup-local script will call
			# ACK! I _hate_ "unless ( ... )". Why can't folks use "if (! ...)" ???
			# Gee, Larry, let's make another function that's Perl-specific. Gag!
			unless ($GLOBAL_LOGONLY) {
				`mv /sbin/ifup-local /sbin/ifup-local.pre-bastille`;
			}
		}
      }
      &B_open(*IFUP_TEMPLATE,"ifup-local");
      &B_open(*IFUP_SCRIPT,">/sbin/ifup-local");
      while ($line=<IFUP_TEMPLATE>) {
         &B_print(*IFUP_SCRIPT,$line);
      }
      &B_close(*IFUP_TEMPLATE);
      &B_close(*IFUP_SCRIPT);
      &B_chmod(0500, '/sbin/ifup-local');
      
}

1;

